"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.error = exports.warn = exports.debug = void 0;
const chalk_1 = require("chalk");
const util_1 = require("util");
const LEVEL_COLORS = {
    debug: chalk_1.green,
    warn: chalk_1.yellow,
    error: chalk_1.red,
};
function label(level, message) {
    return `[@formatjs/ts-transformer] [${LEVEL_COLORS[level](level.toUpperCase())}] ${message}`;
}
async function debug(message, ...args) {
    if (process.env.LOG_LEVEL !== 'debug') {
        return;
    }
    console.error((0, util_1.format)(label('debug', message), ...args));
    console.error('\n');
}
exports.debug = debug;
function warn(message, ...args) {
    console.error((0, util_1.format)(label('warn', message), ...args));
    console.error('\n');
}
exports.warn = warn;
function error(message, ...args) {
    console.error((0, util_1.format)(label('error', message), ...args));
    console.error('\n');
}
exports.error = error;
